''' mi_parser.py
The parser module, part of the getting-and-setting package contains functions
to ease the conversion of Infor ION API responses by convering XML to python
objects.

The reconmended way to use this module is by using the parse_mi_xml factory function
to generate a MiData object that contains relevant parameters such as metadata, records,
program and transaction.

Author: Kim Timothy Engh
Email: kim.timothy.engh@epiroc.com
Licence: GPLv3. See ../LICENCE '''

from typing import Union
from functools import cache
from xml.etree import ElementTree

from .mi_models import MiRecords
from .mi_models import MiPrograms
from .mi_models import MiProgramMetadata
from .mi_models import MiTransactionMetadata
from .mi_models import MiFieldMetadata
from .mi_models import MiApiError


@cache
def parse_xml(xml_str: Union[str, bytes]) -> ElementTree.ElementTree:
    ''' Parses a xml string and returns the element tree. '''
    xml_element_tree = ElementTree.ElementTree(ElementTree.fromstring(xml_str))
    return xml_element_tree

@cache
def xml_ns_tag(xml_str: Union[str, bytes]) -> str:
    ''' Returns the XML name space as a string'''
    xml_ns_tag = parse_xml(xml_str).getroot().tag.split('}')[0] + '}'
    return xml_ns_tag


def has_mi_error(xml_str: str) -> Union[MiApiError, None]:
    ''' Checks if the xml has error messages from the api,
    and returns a MiError object if it exits '''
    xml_element_tree = parse_xml(xml_str).getroot()

    if xml_element_tree.tag == (xml_ns_tag(xml_str) + 'ErrorMessage'):
        mi_error_message = str(xml_element_tree.find(xml_ns_tag(xml_str) + 'Message').text)
        mi_error_type = str(xml_element_tree.get('type'))

        return MiApiError(mi_error_type, mi_error_message, xml_str)

    else:
        return None


def mi_parse_execute(xml_str: str) -> MiRecords:
    '''
    Parses a MI xml string and returns a MiData dataclass.
    If an error is detected in the result, then a MiError
    object is returned instead.
    '''
    xml_root = parse_xml(xml_str).getroot()
    miApiError = has_mi_error(xml_str)

    if miApiError:
        raise miApiError

    return MiRecords(
        program = xml_root.find(xml_ns_tag(xml_str) + 'Program').text,
        transaction = xml_root.find(xml_ns_tag(xml_str) + 'Transaction').text,
        metadata = [
           child.attrib for child
            in xml_root.find(xml_ns_tag(xml_str) + 'Metadata').iter()
            if child.attrib
        ],
        records = [
           {
                mi_name_value.find(xml_ns_tag(xml_str) + 'Name').text:
                mi_name_value.find(xml_ns_tag(xml_str) + 'Value').text
                for mi_name_value in mi_record.findall(xml_ns_tag(xml_str) + 'NameValue')
            }
            for mi_record in xml_root.findall(xml_ns_tag(xml_str) + 'MIRecord')
        ]
    )


def mi_parse_programs(xml_str: Union[str, bytes]) -> MiPrograms:
    ''' Parse a MI xml from a progam call. Returns a MiPrograms dataclass that
    contains the list of program names. '''
    xml_root = parse_xml(xml_str).getroot()

    records = [
        name.text for name in
        xml_root.findall(xml_ns_tag(xml_str) + 'Name')
    ]

    return MiPrograms(records)


def mi_parse_metadata(xml_str: Union[str, bytes]) -> MiProgramMetadata:
    mi_error = has_mi_error(xml_str)

    if mi_error:
        raise mi_error

    xml_root = parse_xml(xml_str).getroot()
    xml_ns = xml_ns_tag(xml_str)

    miProgramMetadata = MiProgramMetadata(
        program = xml_root.attrib['Program'],
        description = xml_root.attrib['Description'],
        version = xml_root.attrib['Version'],
        transactions = [
            MiTransactionMetadata(
                program=mi_transaction.attrib['Program'],
                transaction=mi_transaction.attrib['Transaction'],
                description=mi_transaction.attrib['Description'],
                multi=mi_transaction.attrib['Multi'],
                outputs=[
                    MiFieldMetadata(
                        name=mi_field.attrib['Name'],
                        description=mi_field.attrib['Description'],
                        fieldtype=mi_field.attrib['FieldType'],
                        length=mi_field.attrib['Length'],
                        mandatory=mi_field.attrib['Mandatory']
                    )
                    for mi_field
                    in mi_transaction.find(xml_ns + 'OutputFieldList')
                ],
                inputs=[
                    MiFieldMetadata(
                        name=mi_field.attrib['Name'],
                        description=mi_field.attrib['Description'],
                        fieldtype=mi_field.attrib['FieldType'],
                        length=mi_field.attrib['Length'],
                        mandatory=mi_field.attrib['Mandatory']
                    )
                    for mi_field
                    in mi_transaction.find(xml_ns + 'InputFieldList')
                ]
            )
            for mi_transaction in xml_root.findall(xml_ns + 'Transaction') 
        ]
    )
    
    return miProgramMetadata


''' mi_api.py
A module to that makes calles to the Infor ION API simple. This moduele
handels url creation, authentication and requests. Reconmended way to use
this moduel is by using the class Api.

Author: Kim Timothy Engh
Email: kim.timothy.engh@epiroc.com
Licence: GPLv3 '''


import requests
from requests.auth import HTTPBasicAuth
from typing import Optional


def dict_to_param_str(param_dict: dict) -> str:
    '''
    Convert a dict with key values and return a parameter string to
    be used as parameters in a request url. Note that the argument
    max_recs is treaded as a special case and is added in the string
    before the search parameters.

    The function supports two special keywords that are treated differently.
        1. maxrecs
        2. returncols

    maxrecs sets the number of return records (normally limited to 100 records).
    returncols sets the columns to be returned by the api.

    >>> dict_to_param_str({'EDES': 'AU1'})
    '?EDES=AU1'

    >>> dict_to_param_str({'WHSL': 'AUA', 'ROUT': 'AA0001'})
    '?WHSL=AUA&ROUT=AA0001'

    >>> dict_to_param_str({'max_recs': 0, 'WHSL': 'AUA', 'ROUT': 'AA0001'})
    ';maxrecs=0;?WHSL=AUA&ROUT=AA0001'

    >>> dict_to_param_str({'max_rex': 10, returncols='WHLO,ITNO,EOQM', 'ITNO': 3222148800})
    ';maxrecs=0;returncols='WHLO,ITNO,EOQM'?ITNO=3222148800'
    '''

    maxrecs = f';maxrecs={param_dict.pop("maxrecs")};' if param_dict.get("maxrecs") != None else ''
    returncols = f';returncols={param_dict.pop("returncols")};' if param_dict.get("returncols") != None else ''

    params_str = r'&'.join(
        [
            f'{key}={value}'
            for key, value
            in param_dict.items()
        ]
    )

    return f'{maxrecs}{returncols}?{params_str}'


def mi_request(url: str, usr: str, pwd: str) -> requests.models.Response:
    ''' Returns a request object for any valid url in the API
    :param url - The complete url with parameters.
    :param port - The host port
    :param usr - The user name
    :param pwd - The user password'''
    with requests.get(url, verify=False, auth=HTTPBasicAuth(usr, pwd)) as request:
        return request


def mi_request_metadata(host: str, port: int, usr: str, pwd: str, program: str) -> requests.models.Request:
    ''' Returns a Request object with metadata for program and transactions
    :param host - The host ip address
    :param port - The host port
    :param usr - The user name
    :param pwd - The user password'''

    url = f'http://{host}:{port}/m3api-rest/metadata/{program}'
    request = mi_request(url, usr, pwd)

    return request


def mi_request_programs(host: str, port: int, usr: str, pwd: str) -> requests.models.Request:
    ''' Request a list of program names availible in the API

    :param host - The host ip address
    :param port - The host port
    :param usr - The user name
    :param pwd - The user password'''
    url = f'http://{host}:{port}/m3api-rest/metadata'
    request = mi_request(url, usr, pwd)

    return request

def mi_request_execute(host: str, port: int, usr: str, pwd:str, program: str, transaction: str, **kwargs: Optional[dict]) -> requests.models.Response:
    ''' Executes an API api call and returns a Request object. The Request.content attribute will
    the payload in XML that can be parsed.
    
    :param program - The program name
    :param transaction - The program transaction
    :param **kwargs - Keyword arguments to be passed to the API
        The maxrecs keyword will set the maximum number of return records.
        maxrecs=0 will return all records'''

    param_str = dict_to_param_str(kwargs)
    url = f'http://{host}:{port}/m3api-rest/execute/{program}/{transaction}{param_str}'
    request = mi_request(url, usr, pwd)

    return request



from openpyxl import Workbook
from pathlib import Path

from .mi_models import MiTransactionMetadata
from .mi_endpoints import mi_endpoint_list

def _dtyp(fieldtype: str) -> str:
    if fieldtype == 'Alpha':
        return 'A'
    elif fieldtype in ('Numeric', 'Number', 'Integer'):
        return 'S'
    elif fieldtype == 'Date':
        return 'A'
    else:
        raise LookupError(f'Fieldtype {fieldtype} is not defined')


def mi_output_generate_template(data: MiTransactionMetadata, path: str) -> None:
    wb = Workbook()

    ws = wb.active
    ws.title = f'API_{data.program}_{data.transaction}'

    ws.append(['MESSAGE'] + [field.name for field in data.inputs])
    ws.append(['MESSAGE'] + [field.description for field in data.inputs])
    for _ in range(4): ws.append([''])
    ws.append([''] + [f'{_dtyp(field.fieldtype)}.{field.length}' for field in data.inputs])

    wb.create_sheet('MWS')
    ws = wb['MWS']

    ws.append(
        [
            "Application/Environment Name",
            "Web Services Homepage or REST Server Address(<Server>:<Port>)",
            "Service Context/ and/or 'm3api-rest'",
            "optional: Customer Domain or Server",
            "optional: Customer Domain Key"
        ]
    )
    for endpoint in mi_endpoint_list():
        ws.append(
            [
                endpoint.name,
                f'http://{endpoint.host}:{endpoint.port}/mws/',
                'services|m3api-rest'
            ]
        )

    wb.save(path.absolute().joinpath(Path(f'Template_M3_{data.program}_{data.transaction}.xlsx')))



import typer
import tabulate
import enum

from typing import Optional, List
from pathlib import Path

from . mi_api import mi_api_execute
from . mi_api import mi_meta

from . mi_endpoints import mi_endpoint_get
from . mi_endpoints import mi_endpoint_delete
from . mi_endpoints import mi_endpoint_get
from . mi_endpoints import mi_endpoint_list
from . mi_endpoints import mi_endpoint_save

from . mi_models import Endpoint, MiRecords
from . mi_models import MiProgramMetadata
from . mi_models import MiPrograms
from . mi_models import MiTransactionMetadata

from . mi_outputs import mi_output_generate_template


Keys = enum.Enum(
    'Keys',
    {
        field: field
        for field
        in Endpoint.__dataclass_fields__.keys()
    }
)

Names = enum.Enum(
    'Names',
    {
        endpoint.name: endpoint.name
        for endpoint
        in mi_endpoint_list()
    }
)

app = typer.Typer()


@app.command()
def execute(endpoint: str, program: str, transaction: str, params: Optional[List[str]] = typer.Argument(None)):
    '''Excecute API calls and print the results '''
    try:
        params =  {
            f'{row.split("=")[0]}': row.split('=')[1]
            for row in params
        }
    except:
        typer.echo(f'Failed to parse extra parameters!\n {params}\n Has to have format KEY=VALUE')
        return
    
    records = mi_api_execute(endpoint, program, transaction, **params)

    if isinstance(records, MiRecords):
        print('\n')
        print(
            tabulate.tabulate(
                [
                    record for record
                    in records.records
                ], headers='keys'
            )
        )
    else:
        print(records)


@app.command()
def template(endpoint_name: Names, program: str, transaction: str, path: Optional[Path] = typer.Argument(None)):
    '''Generate template file for SmartDataTool'''
    if not path:
        path = Path("~/")
    elif not path.absolute().is_dir():
        print("Enter a path only")
        return

    endpoint = mi_endpoint_get(endpoint_name.value)
    mi_output_generate_template(
        mi_meta(
            endpoint_name.value,
            program,
            transaction
        ),
        path.absolute()
    )

@app.command()
def endpoint(name: Optional[str] = typer.Argument(None), delete: Optional[bool] = None):
    ''' List or configure the API endpoints '''
    if not name:
        print(
            tabulate.tabulate(
                [
                    {
                        key: val for (key, val) in endpoint.__dict__.items()
                        if key != 'pwd'
                    }
                    for endpoint
                    in mi_endpoint_list()
                ],
                headers='keys'
            )
        )
        return

    if delete:
        confirmation = typer.confirm(f'Really delete {name}?')
        if confirmation:
            mi_endpoint_delete(name)
            typer.echo(f'Endpoint {name} is deleted!')
            return

    try:
        endpoint = mi_endpoint_get(name)
        typer.echo('Updating existing endpoint')

    except ValueError:
        typer.echo('Setting up new endpoint')
        endpoint = None

    if not endpoint:
        endpoint = Endpoint(
            name=name,
            host=typer.prompt('Enter host'),
            port=typer.prompt('Enter port'),
            usr=typer.prompt('Enter user'),
            pwd=typer.prompt(
                'Enter password',
                hide_input=True
            )
        )

    else:
        endpoint = Endpoint(
            name=name,
            host=typer.prompt('Enter host', default=endpoint.host),
            port=typer.prompt('Enter port', default=endpoint.port),
            usr=typer.prompt('Enter user', default=endpoint.usr),
            pwd=typer.prompt(
                'Enter password',
                default=endpoint.pwd,
                hide_input=True,
                show_default=False
            )
        )

    mi_endpoint_save(endpoint)
    typer.echo(f'Saved {name}!')

@app.command()
def meta(endpoint: Names, program: Optional[str] = typer.Argument(None), transaction: Optional[str] = typer.Argument(None)):
    metadata = mi_meta(endpoint.value, program, transaction)

    if isinstance(metadata, MiPrograms):
        for row in metadata.records:
            typer.echo(row)

    elif isinstance(metadata, MiProgramMetadata):
        typer.echo(f'{metadata.program} - {metadata.description}\n')
    
        print(
            tabulate.tabulate(
                [
                    {
                        'transaction': transaction.transaction,
                        'description': transaction.description
                    }
                    for transaction in
                    metadata.transactions
                ],
                headers='keys'
            )
        )
    elif isinstance(metadata, MiTransactionMetadata):
        typer.echo(f'{metadata.program}/{metadata.transaction}')
        typer.echo('\nInputs:')
        print(
            tabulate.tabulate(
                [
                    input.__dict__ for input in
                    metadata.inputs
                ],
                headers='keys'
            )
        )   

        typer.echo('\nOutputs:')
        print(
            tabulate.tabulate(
                [
                    output.__dict__ for output in
                    metadata.outputs
                ],
                headers='keys'
            )
        )
        
    else:
        raise Exception('Undefined return type, {type(metadata)}')

if __name__ == '__main__':
    app()


''' mi_models.py
A module to that makes calles to the Infor ION API simple. This module
contains dataclasses that represents data objects from the Api.

Author: Kim Timothy Engh
Email: kim.timothy.engh@epiroc.com
Licence: GPLv3 '''

from dataclasses import dataclass, field
from typing import Callable, Dict, List, Optional, Iterator, Any


@dataclass
class Endpoint:
    name: str
    host: str
    port: int
    usr: str
    pwd: str = field(default='')

    def __repr__(s):
        return f'Endpoint(name={s.name}, host={s.host}, port={s.port}, usr={s.usr}, pwd={"*" * len(s.pwd)})'


@dataclass
class MiRecords:
    program: str
    transaction: str
    metadata: List[Dict] = field(repr=False)
    records: List[Dict] = field(repr=False)
    
    @staticmethod
    def __numeric_to_int_float(value: str):
        try:
            return int(value.strip())
        
        except ValueError:
            pass
        
        try:
            return float(value.strip())
        
        except ValueError:
            pass
        
        if value == '':
            return None
        
        else:
            raise ValueError(f'Could not convert value {value} of type {type(value)} to int, float or None')

    @staticmethod
    def __alpha_to_str(value: str):
        return value
        
    @staticmethod
    def __date_to_int(value: str):
        try:
            return int(value.strip())

        except ValueError:
            pass
        
        if value == '':
            return None

    def converted(self, strip=False) -> Iterator[Dict]:
        """ Converts values to correct datatypes based on the metadata
        in the response from the API.
        
        [A, 6] '      '   -> '      '
        [A, 6] 'ABC   '   -> 'ABC'
        [A, 6] 'ABCDEF'   -> 'ABCDEF'
        [A, 6] ''         -> ''
        [N, 1] '1'        -> 1
        [N, 2] ' 1'       -> 1
        [N, 2] '99'       -> 99
        [N, 2] '1.2'      -> 1.2
        [N, 2] ''         -> None
        [D, 8] '20220201' -> 20220201
        [D, 8] ''         -> None
        
        """
        
        converters: List[Callable[[str], Any]] = []
        
        for field in self.metadata:
            if field['type'] == 'A':
                if strip:
                    converters.append(lambda x: self.__alpha_to_str(x.strip()))
                  
                else:  
                    converters.append(self.__alpha_to_str)
            
            elif field['type'] == 'N':
                converters.append(self.__numeric_to_int_float)
                
            elif field['type'] == 'D':
                converters.append(self.__date_to_int)
                
            else:
                print(f''' No converter function for type "{field['type']}" ''')
                converters.append(lambda x: x)
        
        def convert_record(record: Dict[str, str]):
            for (key, value), converter in zip(record.items(), converters):                
                try:
                    record[key] = converter(value)
                
                except:       
                    print(f"""Could not convert key "{key}" with value "{value}" from record {record} """)

            return record
    
        for record in self.records:
            yield convert_record(record)


@dataclass
class MiPrograms:
    records: List[str]


@dataclass
class MiFieldMetadata:
    name: str
    description: str
    fieldtype: str
    length: int
    mandatory: str


@dataclass
class MiTransactionMetadata:
    program: str
    transaction: str
    description: str
    multi: str
    inputs: List[MiFieldMetadata] = field(repr=False)
    outputs: List[MiFieldMetadata] = field(repr=False)


@dataclass
class MiProgramMetadata:
    program: str
    description: str
    version: str
    transactions: List[Optional[MiTransactionMetadata]] = field(repr=False)


class MiApiError(Exception):
    """ Error class that returns exceptions raised by the the service.
    This comes handy, when incorrect paramters are entered, or if there
    are any other usage errors. """
    
    def __init__(self, code: str, desc: str, xml: str):
        self.code = code
        self.desc = desc
        self.xml = xml
        
        super(MiApiError, self).__init__(f"MiError({': '.join([code, desc])})")
        
        



import configparser
import keyring
import pathlib
from typing import Dict, List, Union
from functools import cache

from . mi_models import Endpoint


PATH = str(pathlib.Path.joinpath(pathlib.Path.home(), '.getting-and-setting.ini').absolute())
SERVICE_ID = 'getting-and-setting'


def mi_endpoint_save(endpoint: Endpoint) -> Endpoint:
    config = configparser.ConfigParser()
    config.read(PATH)

    config[f'endpoint.{endpoint.name}'] = {
        'host': endpoint.host,
        'port': endpoint.port,
        'usr': endpoint.usr
    }

    with open(PATH, 'w') as f:
        config.write(f)

    keyring.set_password(SERVICE_ID, endpoint.name, endpoint.pwd)

    return mi_endpoint_get(endpoint.name)

@cache
def mi_endpoint_get(name: str) -> Endpoint:
    config = configparser.ConfigParser()
    config.read(PATH)

    try:
        endpoint = dict(config[f'endpoint.{name}'])

    except KeyError:
        raise ValueError(f'Endpoint {name} not found in configuration')

    endpoint['pwd'] = keyring.get_password(SERVICE_ID, name)

    return Endpoint(name=name, **endpoint)


def mi_endpoint_delete(name: str) -> None:
    config = configparser.ConfigParser()
    config.read(PATH)

    config.remove_section(f'endpoint.{name}')

    with open(PATH, 'w') as f:
        config.write(f)

    try:
        keyring.delete_password(SERVICE_ID, name)

    except keyring.core.backend.errors.PasswordDeleteError:
        pass


def mi_endpoint_list() -> List[Endpoint]:
    config = configparser.ConfigParser()
    config.read(PATH)

    return [
        Endpoint(
            name = config[section].name.split('.')[1],
            host = config[section]['host'],
            port = config[section]['port'],
            usr = config[section]['usr'],
            pwd = keyring.get_password(
                SERVICE_ID,
                config[section].name.split('.')[1]
            )
        )
        for section
        in config.sections()
        if section.startswith('endpoint')
    ]


''' mi_api.py
The mi_api file, part of the getting-and-setting package contains functions
combines functions from parser.py and request.py to provide abstract tools to
work with the MI Api.

Author: Kim Timothy Engh
Email: kim.timothy.engh@epiroc.com
Licence: GPLv3. See ../LICENCE '''

from typing import Union
from typing import Optional

from functools import lru_cache

from . mi_models import MiRecords
from . mi_models import MiPrograms
from . mi_models import MiProgramMetadata
from . mi_models import MiTransactionMetadata

from . mi_parser import mi_parse_execute
from . mi_parser import mi_parse_metadata
from . mi_parser import mi_parse_programs

from . mi_request import mi_request_metadata
from . mi_request import mi_request_execute
from . mi_request import mi_request_programs

from . mi_endpoints import mi_endpoint_get


class MiApi:
    def __init__(self, host: str, port: int, usr: str, pwd: str):
        self.host = host
        self.port = port
        self.usr = usr
        self.pwd = pwd
        
    @staticmethod
    def from_env(name: str) -> 'MiApi':
        endpoint = mi_endpoint_get(name)
        return MiApi(
            endpoint.host,
            endpoint.port,
            endpoint.usr,
            endpoint.pwd
        )

    def execute(self, program: str, transaction: str, **kwargs) -> MiRecords:
        return mi_parse_execute(
            mi_request_execute(
                self.host,
                self.port,
                self.usr,
                self.pwd,
                program,
                transaction,
                **kwargs
            ).content
        )

    def program_list(self) -> MiPrograms:
        return mi_parse_programs(
            mi_request_programs(
                self.host,
                self.port,
                self.usr,
                self.pwd
            ).content
        )

    def program_meta(self, program: str) -> MiProgramMetadata:
        return mi_parse_metadata(
            mi_request_metadata(
                self.host,
                self.port,
                self.usr,
                self.pwd,
                program
            ).content
        )

    def transaction_meta(self, program: str, transaction: Optional[str]) -> Union[MiTransactionMetadata, None]:
        program_meta = self.program_meta(program)

        for record in program_meta.transactions:
            if record.transaction == transaction:
                return record
        else:
            return None


@lru_cache(100)
def mi_meta(endpoint: str, program: Optional[str] = None, transaction: Optional[str] = None) -> Union[MiPrograms, MiProgramMetadata, MiTransactionMetadata]:
    ''' Fetch metatdata from the API. Wrapper function for the following functions:
    - mi_meta_list
    - mi_meta_program
    - mi_meta_transactions

    :param endpoint - the name of the enpoint
    :param program - M3 program name
    :param transaction - Program transaction

    Valid combinations of arguments are:
    
    case1: endpoint and not program and not transaction -> MiPrograms
    case2: endpoint and program and not transaction -> MiProgramMetadata
    case3: endpoint and program and transaction -> MiTransactionMetadata

    Other combinations will result in error.'''

    if not program and not transaction:
        return mi_parse_programs(
            mi_request_programs(
                mi_endpoint_get(endpoint).host,
                mi_endpoint_get(endpoint).port,
                mi_endpoint_get(endpoint).usr,
                mi_endpoint_get(endpoint).pwd
            ).content
        )

    elif program and not transaction:
        return  mi_parse_metadata(
                    mi_request_metadata(
                        mi_endpoint_get(endpoint).host,
                        mi_endpoint_get(endpoint).port,
                        mi_endpoint_get(endpoint).usr,
                        mi_endpoint_get(endpoint).pwd,
                        program
                    ).content
                )

    elif program and transaction:
        return [
            row for row in mi_parse_metadata(
                mi_request_metadata(
                    mi_endpoint_get(endpoint).host,
                    mi_endpoint_get(endpoint).port,
                    mi_endpoint_get(endpoint).usr,
                    mi_endpoint_get(endpoint).pwd,
                    program
                ).content
            ).transactions
            if row.transaction == transaction
        ][0]

    else:
        raise RuntimeError('No case defined for program={program} and transaction={transaction}')


def mi_api_execute(endpoint: str, program: str, transaction: str, **kwargs) -> MiRecords:
    ''' Executes API calls, either read or write depending on the transaction.
    The **kwargs represents the input values. The key word "maxrecs" is treated as a special case
    and represents the maximum number of return records. Default is 100, 0 will be all. the key word
    "returncols" is a string with comma seperated column names, that defines which coulumns will
    be returned.
    
    :param endpoint - The name of the endpoint
    :param program - The M3 program
    :param metadata - The program transactions
    :param **kwargs - The input for the transaction '''

    return mi_parse_execute(
        mi_request_execute(
            mi_endpoint_get(endpoint).host,
            mi_endpoint_get(endpoint).port,
            mi_endpoint_get(endpoint).usr,
            mi_endpoint_get(endpoint).pwd,
            program,
            transaction,
            **kwargs
        ).content
    )




def get_api_for_endpoint(name: str) -> MiApi:
    ''' Creates a MiApi object for a stored endpoint'''
    print('get_api_for_endpoint will be depreciated in future release.')
    endpoint = mi_endpoint_get(name)
    return MiApi(
        endpoint.host,
        endpoint.port,
        endpoint.usr,
        endpoint.pwd
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['getting_and_setting']

package_data = \
{'': ['*']}

install_requires = \
['keyring>=23.5.1,<24.0.0',
 'requests>=2.27.1,<3.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['micli = getting_and_setting.mi_cli:app']}

setup_kwargs = {
    'name': 'getting-and-setting',
    'version': '0.5.5',
    'description': 'An interface to interact with the Ion Mi Api.',
    'long_description': None,
    'author': 'Kim Timothy Engh',
    'author_email': 'kimothy@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
